@@ -1,8 +1,12 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE RecordWildCards #-}
+#if __GLASGOW_HASKELL__ >= 800
+{-# LANGUAGE TemplateHaskellQuotes #-}
+#else
 {-# LANGUAGE TemplateHaskell #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
+#endif
+#if __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
 #endif
 {-# LANGUAGE TypeFamilies #-}
@@ -112,11 +116,20 @@ derivingUnbox name argsQ toRepQ fromRepQ = do
         ArrowT `AppT` typ `AppT` rep -> return ([], typ, rep)
         _ -> fail "Expecting a type of the form: cxts => typ -> rep"
 
-    s <- VarT <$> newName "s"
-    let newtypeMVector = NewtypeInstD [] ''MVector [s, typ]
-            (NormalC mvName [(NotStrict, ConT ''MVector `AppT` s `AppT` rep)]) []
+    let s = VarT (mkName "s")
+#if MIN_VERSION_template_haskell(2,11,0)
+    let lazy = Bang NoSourceUnpackedness NoSourceStrictness
+# define MAYBE_KIND Nothing
+# define MAYBE_OVERLAP Nothing
+#else
+    let lazy = NotStrict
+# define MAYBE_KIND
+# define MAYBE_OVERLAP
+#endif
+    let newtypeMVector = NewtypeInstD [] ''MVector [s, typ] MAYBE_KIND
+            (NormalC mvName [(lazy, ConT ''MVector `AppT` s `AppT` rep)]) []
     let mvCon = ConE mvName
-    let instanceMVector = InstanceD cxts
+    let instanceMVector = InstanceD MAYBE_OVERLAP cxts
             (ConT ''M.MVector `AppT` ConT ''MVector `AppT` typ) $ concat
             [ wrap 'M.basicLength           [mv]        id
             , wrap 'M.basicUnsafeSlice      [i, n, mv]  (AppE mvCon)
@@ -134,10 +147,10 @@ derivingUnbox name argsQ toRepQ fromRepQ = do
             , wrap 'M.basicUnsafeMove       [mv, mv']   id
             , wrap 'M.basicUnsafeGrow       [mv, n]     (liftE mvCon) ]
 
-    let newtypeVector = NewtypeInstD [] ''Vector [typ]
-            (NormalC vName [(NotStrict, ConT ''Vector `AppT` rep)]) []
+    let newtypeVector = NewtypeInstD [] ''Vector [typ] MAYBE_KIND
+            (NormalC vName [(lazy, ConT ''Vector `AppT` rep)]) []
     let vCon  = ConE vName
-    let instanceVector = InstanceD cxts
+    let instanceVector = InstanceD MAYBE_OVERLAP cxts
             (ConT ''G.Vector `AppT` ConT ''Vector `AppT` typ) $ concat
             [ wrap 'G.basicUnsafeFreeze     [mv]        (liftE vCon)
             , wrap 'G.basicUnsafeThaw       [v]         (liftE mvCon)
@@ -147,7 +160,7 @@ derivingUnbox name argsQ toRepQ fromRepQ = do
             , wrap 'G.basicUnsafeCopy       [mv, v]     id
             , wrap 'G.elemseq               [v, a]      id ]
 
-    return [ InstanceD cxts (ConT ''Unbox `AppT` typ) []
+    return [ InstanceD MAYBE_OVERLAP cxts (ConT ''Unbox `AppT` typ) []
         , newtypeMVector, instanceMVector
         , newtypeVector, instanceVector ]
 
